@@ -1,13 +1,15 @@
 #![crate_type = "dylib"]
+#![feature(plugin)]
+#![plugin(interpolate_idents)]
 
 #[macro_use] extern crate cpython;
 
-use cpython::{PyObject, PyResult, PyModule, Python, PyTuple};
+use cpython::{PyObject, PyResult,Python, PyTuple};
 
-py_module_initializer!("hello", inithello, |py, m| {
+py_module_initializer!(hello, |_py, m| {
     try!(m.add("__doc__", "Module documentation string"));
-    try!(m.add("run", py_func!(py, run)));
-    try!(add_val(py, &m));
+    try!(m.add("run", py_fn!(run)));
+    try!(m.add("val", py_fn!(val)));
     Ok(())
 });
 
@@ -23,8 +25,3 @@ fn val<'p>(_: Python<'p>, _: &PyTuple<'p>) -> PyResult<'p, i32> {
     Ok(42)
 }
 
-// Workaround for Rust #24561
-fn add_val<'p>(py: Python<'p>, m: &PyModule<'p>) -> PyResult<'p, ()> {
-    m.add("val", py_func!(py, val))
-}
-
