@@ -19,13 +19,14 @@
 use std;
 use python::{PythonObject, Python, ToPythonPointer, PythonObjectDowncastError};
 use objects::{PyObject, PyType, exc};
+#[cfg(feature="python27-sys")]
 use objects::oldstyle::PyClass;
 use ffi;
 use libc;
 use conversion::ToPyObject;
-use std::ffi::{CString, CStr};
+use std::ffi::CString;
 
-/// Represents a python exception that was raised.
+/// Represents a Python exception that was raised.
 #[derive(Clone, Debug)]
 pub struct PyErr<'p> {
     /// The type of the exception. This should be either a `PyClass` or a `PyType`.
@@ -42,18 +43,18 @@ pub struct PyErr<'p> {
 }
 
 
-/// Represents the result of a python call.
+/// Represents the result of a Python call.
 pub type PyResult<'p, T> = Result<T, PyErr<'p>>;
 
 impl <'p> PyErr<'p> {
-    /// Gets whether an error is present in the python interpreter's global state.
+    /// Gets whether an error is present in the Python interpreter's global state.
     #[inline]
     pub fn occurred(_ : Python<'p>) -> bool {
         unsafe { !ffi::PyErr_Occurred().is_null() }
     }
 
-    /// Retrieves the current error from the python interpreter's global state.
-    /// The error is cleared from the python interpreter.
+    /// Retrieves the current error from the Python interpreter's global state.
+    /// The error is cleared from the Python interpreter.
     /// If no error is set, returns a `SystemError`.
     pub fn fetch(py : Python<'p>) -> PyErr<'p> {
         unsafe {
@@ -78,16 +79,16 @@ impl <'p> PyErr<'p> {
             ptraceback: PyObject::from_owned_ptr_opt(py, ptraceback)
         }
     }
-    
+
     /// Creates a new PyErr.
     ///
-    /// If `obj` is a python exception instance, the PyErr will use that instance.
-    /// If `obj` is a python exception type, the PyErr will (lazily) create a new instance of that type
-    /// Otherwise, a `TypeError` is returned instead.
+    /// If `obj` is a Python exception instance, the PyErr will use that instance.
+    /// If `obj` is a Python exception type object, the PyErr will (lazily) create a new instance of that type.
+    /// Otherwise, a `TypeError` is created instead.
     pub fn new<O>(obj: O) -> PyErr<'p> where O: PythonObject<'p> {
         PyErr::new_from_object(obj.into_object())
     }
-    
+
     fn new_from_object(obj: PyObject<'p>) -> PyErr<'p> {
         let py = obj.python();
         if unsafe { ffi::PyExceptionInstance_Check(obj.as_ptr()) } != 0 {
@@ -105,13 +106,13 @@ impl <'p> PyErr<'p> {
         } else {
             PyErr {
                 ptype: py.get_type::<exc::TypeError>().into_object(),
-                pvalue: Some("exceptions must derive from BaseException".to_py_object(py)),
+                pvalue: Some("exceptions must derive from BaseException".to_py_object(py).into_object()),
                 ptraceback: None
             }
         }
     }
 
-    /// Construct a new error, with the usual lazy initialization of python exceptions.
+    /// Construct a new error, with the usual lazy initialization of Python exceptions.
     /// `exc` is the exception type; usually one of the standard exceptions like `PyExc::runtime_error()`.
     /// `value` is the exception instance, or a tuple of arguments to pass to the exception constructor.
     #[inline]
@@ -142,7 +143,7 @@ impl <'p> PyErr<'p> {
     pub fn matches(&self, exc: &PyObject) -> bool {
         unsafe { ffi::PyErr_GivenExceptionMatches(self.ptype.as_ptr(), exc.as_ptr()) != 0 }
     }
-    
+
     /// Normalizes the error. This ensures that the exception value is an instance of the exception type.
     pub fn normalize(&mut self) {
         // The normalization helper function involves temporarily moving out of the &mut self,
@@ -166,9 +167,11 @@ impl <'p> PyErr<'p> {
             PyErr::new_from_ffi_tuple(py, ptype, pvalue, ptraceback)
         }
     }
-    
+
     /// Retrieves the exception type.
+    ///
     /// If the exception type is an old-style class, returns `oldstyle::PyClass`.
+    #[cfg(feature="python27-sys")]
     pub fn get_type(&self) -> PyType<'p> {
         let py = self.ptype.python();
         match self.ptype.clone().cast_into::<PyType>() {
@@ -181,6 +184,16 @@ impl <'p> PyErr<'p> {
         }
     }
 
+    /// Retrieves the exception type.
+    #[cfg(not(feature="python27-sys"))]
+    pub fn get_type(&self) -> PyType<'p> {
+        let py = self.ptype.python();
+        match self.ptype.clone().cast_into::<PyType>() {
+            Ok(t)  => t,
+            Err(_) => py.None().get_type().clone()
+        }
+    }
+
     /// Retrieves the exception instance for this error.
     /// This method takes `&mut self` because the error might need
     /// to be normalized in order to create the exception instance.
@@ -192,7 +205,7 @@ impl <'p> PyErr<'p> {
         }
     }
 
-    /// Writes the error back to the python interpreter's global state.
+    /// Writes the error back to the Python interpreter's global state.
     /// This is the opposite of `PyErr::fetch()`.
     #[inline]
     pub fn restore(self) {
@@ -201,7 +214,7 @@ impl <'p> PyErr<'p> {
             ffi::PyErr_Restore(ptype.steal_ptr(), pvalue.steal_ptr(), ptraceback.steal_ptr())
         }
     }
-    
+
     /// Issue a warning message.
     /// May return a PyErr if warnings-as-errors is enabled.
     pub fn warn(py: Python<'p>, category: &PyObject, message: &str, stacklevel: i32) -> PyResult<'p, ()> {
@@ -212,14 +225,14 @@ impl <'p> PyErr<'p> {
     }
 }
 
-/// Converts `PythonObjectDowncastError` to python `TypeError`.
+/// Converts `PythonObjectDowncastError` to Python `TypeError`.
 impl <'p> std::convert::From<PythonObjectDowncastError<'p>> for PyErr<'p> {
     fn from(err: PythonObjectDowncastError<'p>) -> PyErr<'p> {
         PyErr::new_lazy_init(err.0.get_type::<exc::TypeError>(), None)
     }
 }
 
-/// Construct PyObject from the result of a python FFI call that returns a new reference (owned pointer).
+/// Construct PyObject from the result of a Python FFI call that returns a new reference (owned pointer).
 /// Returns `Err(PyErr)` if the pointer is `null`.
 /// Unsafe because the pointer might be invalid.
 #[inline]
@@ -231,7 +244,7 @@ pub unsafe fn result_from_owned_ptr(py : Python, p : *mut ffi::PyObject) -> PyRe
     }
 }
 
-fn panic_after_error(py: Python) -> ! {
+fn panic_after_error(_py: Python) -> ! {
     unsafe { ffi::PyErr_Print(); }
     panic!("Python API called failed");
 }
@@ -245,18 +258,6 @@ pub unsafe fn from_owned_ptr_or_panic(py : Python, p : *mut ffi::PyObject) -> Py
     }
 }
 
-/// Construct PyObject from the result of a python FFI call that returns a borrowed reference.
-/// Returns `Err(PyErr)` if the pointer is `null`.
-/// Unsafe because the pointer might be invalid.
-#[inline]
-pub unsafe fn result_from_borrowed_ptr(py : Python, p : *mut ffi::PyObject) -> PyResult<PyObject> {
-    if p.is_null() {
-        Err(PyErr::fetch(py))
-    } else {
-        Ok(PyObject::from_borrowed_ptr(py, p))
-    }
-}
-
 pub unsafe fn result_cast_from_owned_ptr<'p, T>(py : Python<'p>, p : *mut ffi::PyObject) -> PyResult<'p, T>
     where T: ::python::PythonObjectWithCheckedDowncast<'p>
 {
@@ -290,8 +291,8 @@ pub fn error_on_minusone(py : Python, result : libc::c_int) -> PyResult<()> {
 #[cfg(test)]
 mod tests {
     use {Python, PyErr};
-    use objects::{PyObject, exc};
-    
+    use objects::exc;
+
     #[test]
     fn set_typeerror() {
         let gil = Python::acquire_gil();
