@@ -16,14 +16,15 @@
 // OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-use std;
 use python::{Python, PythonObject, ToPythonPointer};
-use err::{self, PyResult, PyErr};
+use err::{self, PyResult};
 use super::object::PyObject;
-use super::exc;
 use ffi::{self, Py_ssize_t};
 use conversion::{ToPyObject, FromPyObject};
 
+/// Represents a Python `list`.
+pub struct PyList<'p>(PyObject<'p>);
+
 pyobject_newtype!(PyList, PyList_Check, PyList_Type);
 
 impl <'p> PyList<'p> {
@@ -33,7 +34,7 @@ impl <'p> PyList<'p> {
             let ptr = ffi::PyList_New(elements.len() as Py_ssize_t);
             let t = err::result_from_owned_ptr(py, ptr).unwrap().unchecked_cast_into::<PyList>();
             for (i, e) in elements.iter().enumerate() {
-                ffi::PyList_SET_ITEM(ptr, i as Py_ssize_t, e.clone().steal_ptr());
+                ffi::PyList_SetItem(ptr, i as Py_ssize_t, e.clone().steal_ptr());
             }
             t
         }
@@ -42,9 +43,9 @@ impl <'p> PyList<'p> {
     /// Gets the length of the list.
     #[inline]
     pub fn len(&self) -> usize {
-        // non-negative Py_ssize_t should always fit into Rust uint
+        // non-negative Py_ssize_t should always fit into Rust usize
         unsafe {
-            ffi::PyList_GET_SIZE(self.as_ptr()) as usize
+            ffi::PyList_Size(self.as_ptr()) as usize
         }
     }
 
@@ -54,7 +55,7 @@ impl <'p> PyList<'p> {
     pub fn get_item(&self, index: usize) -> PyObject<'p> {
         assert!(index < self.len());
         unsafe {
-            PyObject::from_borrowed_ptr(self.python(), ffi::PyList_GET_ITEM(self.as_ptr(), index as Py_ssize_t))
+            PyObject::from_borrowed_ptr(self.python(), ffi::PyList_GetItem(self.as_ptr(), index as Py_ssize_t))
         }
     }
 
@@ -65,7 +66,7 @@ impl <'p> PyList<'p> {
         let r = unsafe { ffi::PyList_SetItem(self.as_ptr(), index as Py_ssize_t, item.steal_ptr()) };
         assert!(r == 0);
     }
-    
+
     /// Inserts an item at the specified index.
     ///
     /// Panics if the index is out of range.
@@ -75,6 +76,50 @@ impl <'p> PyList<'p> {
     }
 }
 
+impl <'p> IntoIterator for PyList<'p> {
+    type Item = PyObject<'p>;
+    type IntoIter = PyListIterator<'p>;
+
+    #[inline]
+    fn into_iter(self) -> PyListIterator<'p> {
+        PyListIterator { list: self, index: 0 }
+    }
+}
+
+impl <'a, 'p> IntoIterator for &'a PyList<'p> {
+    type Item = PyObject<'p>;
+    type IntoIter = PyListIterator<'p>;
+
+    #[inline]
+    fn into_iter(self) -> PyListIterator<'p> {
+        PyListIterator { list: self.clone(), index: 0 }
+    }
+}
+
+/// Used by `impl IntoIterator for &PyList`.
+pub struct PyListIterator<'p> {
+    list: PyList<'p>,
+    index: usize
+}
+
+impl <'p> Iterator for PyListIterator<'p> {
+    type Item = PyObject<'p>;
+
+    #[inline]
+    fn next(&mut self) -> Option<PyObject<'p>> {
+        if self.index < self.list.len() {
+            let item = self.list.get_item(self.index);
+            self.index += 1;
+            Some(item)
+        } else {
+            None
+        }
+    }
+
+    // Note: we cannot implement size_hint because the length of the list
+    // might change during the iteration.
+}
+
 impl <'p, T> ToPyObject<'p> for [T] where T: ToPyObject<'p> {
     type ObjectType = PyList<'p>;
 
@@ -84,27 +129,21 @@ impl <'p, T> ToPyObject<'p> for [T] where T: ToPyObject<'p> {
             let t = err::cast_from_owned_ptr_or_panic(py, ptr);
             for (i, e) in self.iter().enumerate() {
                 let obj = e.to_py_object(py);
-                ffi::PyList_SET_ITEM(ptr, i as Py_ssize_t, obj.steal_ptr());
+                ffi::PyList_SetItem(ptr, i as Py_ssize_t, obj.steal_ptr());
             }
             t
         }
     }
 }
 
-/*
- This implementation is not possible, because we allow extracting python strings as CowString<'s>,
- but there's no guarantee that the list isn't modified while the CowString borrow exists.
-impl <'p, 's, T> FromPyObject<'p, 's> for Vec<T> where T: FromPyObject<'p, 's> {
-    fn from_py_object(s: &'s PyObject<'p>) -> PyResult<'p, Vec<T>> {
-        let py = s.python();
+impl <'p, T> FromPyObject<'p> for Vec<T> where T: FromPyObject<'p> {
+    fn from_py_object(s: &PyObject<'p>) -> PyResult<'p, Vec<T>> {
         let list = try!(s.cast_as::<PyList>());
-        let ptr = list.as_ptr();
         let mut v = Vec::with_capacity(list.len());
-        for i in 0..list.len() {
-            let obj = unsafe { PyObject::from_borrowed_ptr(py, ffi::PyList_GET_ITEM(ptr, i as Py_ssize_t)) };
-            v.push(try!(obj.extract::<T>()));
+        for i in 0 .. list.len() {
+            v.push(try!(list.get_item(i).extract::<T>()));
         }
         Ok(v)
     }
 }
-*/
+
