@@ -16,21 +16,55 @@
 // OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
+extern crate num;
+
 use libc::{c_long, c_double};
-use std;
 use python::{Python, PythonObject, ToPythonPointer};
 use err::{self, PyResult, PyErr};
 use super::object::PyObject;
 use super::exc;
-use ffi::{self, Py_ssize_t};
+use ffi;
 use conversion::{ToPyObject, FromPyObject};
 
+/// Represents a Python `int` object.
+///
+/// Note that in Python 2.x, `int` and `long` are different types.
+/// When rust-cpython is compiled for Python 3.x,
+/// `PyInt` and `PyLong` are aliases for the same type, which
+/// corresponds to a Python `int`.
+///
+/// You can usually avoid directly working with this type
+/// by using [ToPyObject](trait.ToPyObject.html)
+/// and [extract](struct.PyObject.html#method.extract)
+/// with the primitive Rust integer types.
+#[cfg(feature="python27-sys")]
+pub struct PyInt<'p>(PyObject<'p>);
+#[cfg(feature="python27-sys")]
 pyobject_newtype!(PyInt, PyInt_Check, PyInt_Type);
+
+/// In Python 2.x, represents a Python `long` object.
+/// In Python 3.x, represents a Python `int` object.
+/// Both `PyInt` and `PyLong` refer to the same type on Python 3.x.
+///
+/// You can usually avoid directly working with this type
+/// by using [ToPyObject](trait.ToPyObject.html)
+/// and [extract](struct.PyObject.html#method.extract)
+/// with the primitive Rust integer types.
+pub struct PyLong<'p>(PyObject<'p>);
 pyobject_newtype!(PyLong, PyLong_Check, PyLong_Type);
+
+/// Represents a Python `float` object.
+///
+/// You can usually avoid directly working with this type
+/// by using [ToPyObject](trait.ToPyObject.html)
+/// and [extract](struct.PyObject.html#method.extract)
+/// with `f32`/`f64`.
+pub struct PyFloat<'p>(PyObject<'p>);
 pyobject_newtype!(PyFloat, PyFloat_Check, PyFloat_Type);
 
+#[cfg(feature="python27-sys")]
 impl <'p> PyInt<'p> {
-    /// Creates a new python `int` object.
+    /// Creates a new Python `int` object.
     pub fn new(py: Python<'p>, val: c_long) -> PyInt<'p> {
         unsafe {
             err::cast_from_owned_ptr_or_panic(py, ffi::PyInt_FromLong(val))
@@ -45,7 +79,7 @@ impl <'p> PyInt<'p> {
 
 
 impl <'p> PyFloat<'p> {
-    /// Creates a new python `float` object.
+    /// Creates a new Python `float` object.
     pub fn new(py: Python<'p>, val: c_double) -> PyFloat<'p> {
         unsafe {
             err::cast_from_owned_ptr_or_panic(py, ffi::PyFloat_FromDouble(val))
@@ -54,12 +88,13 @@ impl <'p> PyFloat<'p> {
 
     /// Gets the value of this float.
     pub fn value(&self) -> c_double {
-        unsafe { ffi::PyFloat_AS_DOUBLE(self.as_ptr()) }
+        unsafe { ffi::PyFloat_AsDouble(self.as_ptr()) }
     }
 }
 
 macro_rules! int_fits_c_long(
     ($rust_type:ty) => (
+        #[cfg(feature="python27-sys")]
         impl <'p> ToPyObject<'p> for $rust_type {
             type ObjectType = PyInt<'p>;
 
@@ -71,14 +106,42 @@ macro_rules! int_fits_c_long(
             }
         }
 
-        impl <'p, 's> FromPyObject<'p, 's> for $rust_type {
-            fn from_py_object(s: &'s PyObject<'p>) -> PyResult<'p, $rust_type> {
+        #[cfg(feature="python3-sys")]
+        impl <'p> ToPyObject<'p> for $rust_type {
+            type ObjectType = PyLong<'p>;
+
+            fn to_py_object(&self, py: Python<'p>) -> PyLong<'p> {
+                unsafe {
+                    err::cast_from_owned_ptr_or_panic(py,
+                        ffi::PyLong_FromLong(*self as c_long))
+                }
+            }
+        }
+
+        #[cfg(feature="python27-sys")]
+        impl <'p> FromPyObject<'p> for $rust_type {
+            fn from_py_object(s: &PyObject<'p>) -> PyResult<'p, $rust_type> {
                 let py = s.python();
                 let val = unsafe { ffi::PyInt_AsLong(s.as_ptr()) };
                 if val == -1 && PyErr::occurred(py) {
                     return Err(PyErr::fetch(py));
                 }
-                match std::num::cast::<c_long, $rust_type>(val) {
+                match num::traits::cast::<c_long, $rust_type>(val) {
+                    Some(v) => Ok(v),
+                    None => Err(overflow_error(py))
+                }
+            }
+        }
+
+        #[cfg(feature="python3-sys")]
+        impl <'p> FromPyObject<'p> for $rust_type {
+            fn from_py_object(s: &PyObject<'p>) -> PyResult<'p, $rust_type> {
+                let py = s.python();
+                let val = unsafe { ffi::PyLong_AsLong(s.as_ptr()) };
+                if val == -1 && PyErr::occurred(py) {
+                    return Err(PyErr::fetch(py));
+                }
+                match num::traits::cast::<c_long, $rust_type>(val) {
                     Some(v) => Ok(v),
                     None => Err(overflow_error(py))
                 }
@@ -99,11 +162,11 @@ macro_rules! int_fits_larger_int(
             }
         }
 
-        impl <'p, 's> FromPyObject<'p, 's> for $rust_type {
-            fn from_py_object(s: &'s PyObject<'p>) -> PyResult<'p, $rust_type> {
+        impl <'p> FromPyObject<'p> for $rust_type {
+            fn from_py_object(s: &PyObject<'p>) -> PyResult<'p, $rust_type> {
                 let py = s.python();
                 let val = try!(s.extract::<$larger_type>());
-                match std::num::cast::<$larger_type, $rust_type>(val) {
+                match num::traits::cast::<$larger_type, $rust_type>(val) {
                     Some(v) => Ok(v),
                     None => Err(overflow_error(py))
                 }
@@ -113,6 +176,84 @@ macro_rules! int_fits_larger_int(
 );
 
 
+fn err_if_invalid_value<'p, T: PartialEq, F: Fn() -> T>
+   (obj: &PyObject<'p>, invalid_value: T, func: F) -> PyResult<'p, T> {
+    let py = obj.python();
+    let v = func();
+    if v == invalid_value && PyErr::occurred(py) {
+        Err(PyErr::fetch(py))
+    } else {
+        Ok(v)
+    }
+}
+
+macro_rules! int_convert_u64_or_i64 (
+    ($rust_type:ty, $pylong_from_ll_or_ull:expr, $pylong_as_ull_or_ull:expr) => (
+        impl <'p> ToPyObject<'p> for $rust_type {
+            #[cfg(feature="python27-sys")]
+            type ObjectType = PyObject<'p>;
+
+            #[cfg(feature="python3-sys")]
+            type ObjectType = PyLong<'p>;
+
+            #[cfg(feature="python27-sys")]
+            fn to_py_object(&self, py: Python<'p>) -> PyObject<'p> {
+                unsafe {
+                    let ptr = match num::traits::cast::<$rust_type, c_long>(*self) {
+                        Some(v) => ffi::PyInt_FromLong(v),
+                        None => $pylong_from_ll_or_ull(*self)
+                    };
+                    err::from_owned_ptr_or_panic(py, ptr)
+                }
+            }
+
+            #[cfg(feature="python3-sys")]
+            fn to_py_object(&self, py: Python<'p>) -> PyLong<'p> {
+                unsafe {
+                    err::cast_from_owned_ptr_or_panic(py, $pylong_from_ll_or_ull(*self))
+                }
+            }
+        }
+
+        impl <'p> FromPyObject<'p> for $rust_type {
+            #[cfg(feature="python27-sys")]
+            fn from_py_object(s: &PyObject<'p>) -> PyResult<'p, $rust_type> {
+                let py = s.python();
+                let ptr = s.as_ptr();
+
+                unsafe {
+                    if ffi::PyLong_Check(ptr) != 0 {
+                        err_if_invalid_value(s, !0, || $pylong_as_ull_or_ull(s.as_ptr()) )
+                    } else if ffi::PyInt_Check(ptr) != 0 {
+                        match num::traits::cast::<c_long, $rust_type>(ffi::PyInt_AS_LONG(ptr)) {
+                            Some(v) => Ok(v),
+                            None => Err(overflow_error(py))
+                        }
+                    } else {
+                        let num = try!(err::result_from_owned_ptr(py, ffi::PyNumber_Long(ptr)));
+                        err_if_invalid_value(&num, !0, || $pylong_as_ull_or_ull(num.as_ptr()) )
+                    }
+                }
+            }
+
+            #[cfg(feature="python3-sys")]
+            fn from_py_object(s: &PyObject<'p>) -> PyResult<'p, $rust_type> {
+                let py = s.python();
+                let ptr = s.as_ptr();
+                unsafe {
+                    if ffi::PyLong_Check(ptr) != 0 {
+                        err_if_invalid_value(s, !0, || $pylong_as_ull_or_ull(s.as_ptr()) )
+                    } else {
+                        let num = try!(err::result_from_owned_ptr(py, ffi::PyNumber_Long(ptr)));
+                        err_if_invalid_value(&num, !0, || $pylong_as_ull_or_ull(num.as_ptr()) )
+                    }
+                }
+            }
+        }
+    )
+);
+
+
 int_fits_c_long!(i8);
 int_fits_c_long!(u8);
 int_fits_c_long!(i16);
@@ -127,9 +268,10 @@ int_fits_larger_int!(u32, u64);
 
 #[cfg(all(target_pointer_width="64", not(target_os="windows")))]
 int_fits_c_long!(i64);
-// TODO: manual implementation for i64 on systems with 32-bit long
 
-// u64 has a manual implementation as it never fits into signed long
+// manual implementation for i64 on systems with 32-bit long
+#[cfg(any(target_pointer_width="32", target_os="windows"))]
+int_convert_u64_or_i64!(i64, ffi::PyLong_FromLongLong, ffi::PyLong_AsLongLong);
 
 #[cfg(all(target_pointer_width="64", not(target_os="windows")))]
 int_fits_c_long!(isize);
@@ -138,49 +280,8 @@ int_fits_larger_int!(isize, i64);
 
 int_fits_larger_int!(usize, u64);
 
-impl <'p> ToPyObject<'p> for u64 {
-    type ObjectType = PyObject<'p>;
-
-    fn to_py_object(&self, py: Python<'p>) -> PyObject<'p> {
-        unsafe {
-            let ptr = match std::num::cast::<u64, c_long>(*self) {
-                Some(v) => ffi::PyInt_FromLong(v),
-                None => ffi::PyLong_FromUnsignedLongLong(*self)
-            };
-            err::from_owned_ptr_or_panic(py, ptr)
-        }
-    }
-}
-
-fn pylong_as_u64<'p>(obj: &PyObject<'p>) -> PyResult<'p, u64> {
-    let py = obj.python();
-    let v = unsafe { ffi::PyLong_AsUnsignedLongLong(obj.as_ptr()) };
-    if v == !0 && PyErr::occurred(py) {
-        Err(PyErr::fetch(py))
-    } else {
-        Ok(v)
-    }
-}
-
-impl <'p, 's> FromPyObject<'p, 's> for u64 {
-    fn from_py_object(s: &'s PyObject<'p>) -> PyResult<'p, u64> {
-        let py = s.python();
-        let ptr = s.as_ptr();
-        unsafe {
-            if ffi::PyLong_Check(ptr) != 0 {
-                pylong_as_u64(s)
-            } else if ffi::PyInt_Check(ptr) != 0 {
-                match std::num::cast::<c_long, u64>(ffi::PyInt_AS_LONG(ptr)) {
-                    Some(v) => Ok(v),
-                    None => Err(overflow_error(py))
-                }
-            } else {
-                let num = try!(err::result_from_owned_ptr(py, ffi::PyNumber_Long(ptr)));
-                pylong_as_u64(&num)
-            }
-        }
-    }
-}
+// u64 has a manual implementation as it never fits into signed long
+int_convert_u64_or_i64!(u64, ffi::PyLong_FromUnsignedLongLong, ffi::PyLong_AsUnsignedLongLong);
 
 impl <'p> ToPyObject<'p> for f64 {
     type ObjectType = PyFloat<'p>;
@@ -190,8 +291,8 @@ impl <'p> ToPyObject<'p> for f64 {
     }
 }
 
-impl <'p, 's> FromPyObject<'p, 's> for f64 {
-    fn from_py_object(s: &'s PyObject<'p>) -> PyResult<'p, f64> {
+impl <'p> FromPyObject<'p> for f64 {
+    fn from_py_object(s: &PyObject<'p>) -> PyResult<'p, f64> {
         let py = s.python();
         let v = unsafe { ffi::PyFloat_AsDouble(s.as_ptr()) };
         if v == -1.0 && PyErr::occurred(py) {
@@ -214,8 +315,8 @@ impl <'p> ToPyObject<'p> for f32 {
     }
 }
 
-impl <'p, 's> FromPyObject<'p, 's> for f32 {
-    fn from_py_object(s: &'s PyObject<'p>) -> PyResult<'p, f32> {
+impl <'p> FromPyObject<'p> for f32 {
+    fn from_py_object(s: &PyObject<'p>) -> PyResult<'p, f32> {
         Ok(try!(s.extract::<f64>()) as f32)
     }
 }
@@ -267,7 +368,29 @@ mod test {
         assert_eq!(v as u64, obj.extract::<u64>().unwrap());
         assert!(obj.extract::<i32>().is_err());
     }
-
+    
+    #[test]
+    fn test_i64_max() {
+        let gil = Python::acquire_gil();
+        let py = gil.python();
+        let v = std::i64::MAX;
+        let obj = v.to_py_object(py).into_object();
+        assert_eq!(v, obj.extract::<i64>().unwrap());
+        assert_eq!(v as u64, obj.extract::<u64>().unwrap());
+        assert!(obj.extract::<u32>().is_err());
+    }
+    
+    #[test]
+    fn test_i64_min() {
+        let gil = Python::acquire_gil();
+        let py = gil.python();
+        let v = std::i64::MIN;
+        let obj = v.to_py_object(py).into_object();
+        assert_eq!(v, obj.extract::<i64>().unwrap());
+        assert!(obj.extract::<i32>().is_err());
+        assert!(obj.extract::<u64>().is_err());
+    }
+    
     #[test]
     fn test_u64_max() {
         let gil = Python::acquire_gil();
@@ -279,4 +402,3 @@ mod test {
         assert!(obj.extract::<i64>().is_err());
     }
 }
-
